part of jaguar_auth.authenticators;

class BasicAuth extends Interceptor {
  /// Model manager used to authenticate, fetch model by authenticationId and
  /// fetch model by authorizationId
  final AuthModelManager modelManager;

  /// The key by which authorizationId shall be stored in session data
  final String authorizationIdKey;

  /// Specifies whether the interceptor should create/update the session data
  /// on authentication success. If set to false, session creation and update
  /// must be done manually
  final bool manageSession;

  BasicAuth(this.modelManager,
      {this.authorizationIdKey: 'id', this.manageSession: true});

  Future<UserBase> pre(Context ctx) async {
    String header = ctx.req.headers.value(HttpHeaders.AUTHORIZATION);
    SessionManager sessionManager = ctx.getInput(SessionInterceptor);
    final AuthHeaderItem basic =
        new AuthHeaderItem.fromHeaderBySchema(header, kBasicAuthScheme);

    if (basic == null) {
      throw new UnAuthorizedError();
    }

    final credentials = _decodeCredentials(basic);
    final usernamePassword = credentials.split(':');

    if (usernamePassword.length != 2) {
      throw new UnAuthorizedError();
    }

    final String username = usernamePassword[0];
    final String password = usernamePassword[1];

    final subject = await modelManager.authenticate(username, password);

    if (subject == null) {
      throw new UnAuthorizedError();
    }

    if (manageSession is bool && manageSession) {
      await sessionManager
          .updateSession({authorizationIdKey: subject.authorizationId});
    }

    return subject;
  }

  String _decodeCredentials(AuthHeaderItem authHeader) {
    try {
      return new String.fromCharCodes(
          const Base64Codec.urlSafe().decode(authHeader.credentials));
    } on FormatException catch (_) {
      return '';
    }
  }

  static const kBasicAuthScheme = 'Basic';
}
