part of jaguar_auth.authenticators;

/// An [Authenticator] for standard username password login using ajax requests.
/// It expects a `application/json` encoded body where the
/// username and password fields must be called `username` and `password`
/// respectively.
class UsernamePasswordJsonAuth extends Interceptor {
  /// Model manager used to authenticate, fetch model by authenticationId and
  /// fetch model by authorizationId
  final AuthModelManager modelManager;

  /// The key by which authorizationId shall be stored in session data
  final String authorizationIdKey;

  /// Specifies whether the interceptor should create/update the session data
  /// on authentication success. If set to false, session creation and update
  /// must be done manually
  final bool manageSession;

  UsernamePasswordJsonAuth(this.modelManager,
      {this.authorizationIdKey: 'id', this.manageSession: true});

  Future<UserBase> pre(Context ctx) async {
    SessionManager sessionManager = ctx.getInput(SessionInterceptor);
    Map<String, dynamic> jsonBody = await ctx.req.bodyAsJsonMap();

    if (jsonBody == null) {
      throw new UnAuthorizedError();
    }

    final String username = jsonBody['username'];
    final String password = jsonBody['password'];

    if (username is! String) {
      throw new UnAuthorizedError();
    }

    if (password is! String) {
      throw new UnAuthorizedError();
    }

    final subject = await modelManager.authenticate(username, password);

    if (subject == null) {
      throw new UnAuthorizedError();
    }

    if (manageSession is bool && manageSession) {
      await sessionManager
          .updateSession({authorizationIdKey: subject.authorizationId});
    }

    return subject;
  }
}
