library jaguar_auth.authoriser;

import 'dart:async';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar_auth/src/entity/entity.dart';
import 'package:jaguar_session/jaguar_session.dart';

class UserAuthoriser extends Interceptor {
  /// Model manager used to authenticate, fetch model by authenticationId and
  /// fetch model by authorizationId
  final AuthModelManager modelManager;

  /// The key by which authorizationId is stored in session data
  final String authorizationIdKey;

  UserAuthoriser(this.modelManager, {this.authorizationIdKey: 'id'});

  Future<UserModel> pre(Context ctx) async {
    final SessionManager sessionManager = ctx.getInput(SessionInterceptor);
    await sessionManager.validate();

    String id = sessionManager.getInValue(authorizationIdKey);
    if (id is! String || id.isEmpty) {
      throw new UnAuthorizedError();
    }

    UserModel subject = await modelManager.fetchModelByAuthorizationId(id);

    if (subject == null) {
      throw new UnAuthorizedError();
    }

    return subject;
  }
  Future<Response> post(Context ctx, Response resp) async {
    final SessionManager sessionManager = ctx.getInput(SessionInterceptor);
    return await sessionManager.updateResponse(ctx.req, resp);
  }
}
